from wstat import Event


ConversationsList = dict()

class Conversation(dict):
    def __init__(self, ID, caller, destination):
        # print("====Init conversation " + str(ID))
        self["ID"] = ID
        self["status"] = "connecting"
        self["caller"] = caller
        self["callee"] = None
        self["app"] = None
        self["destination"] = destination
        self["service"] = None
        self["trunk"] = None
        self["queue"] = None
        self["tags"] = None
        self["flags"] = None
        self["splitReason"] = None
        self["transferType"] = None
        self["mergeWith"] = None
        self["cause"] = None
        self["causeStr"] = None
        self["who"] = None
        self["recording"] = None
        self["lookupInfo"] = None
        self["event"] = Event.Event(ID)
        self["event"]["status"] = self.get("status")
        self["event"]["caller"] = self.get("caller").get_info()
        self["event"]["destination"] = self.get("destination")


    def set(self, param, value):
        #print("\n\n>>> Conversation.set, param: {}, value: {}".format(param,value))
        if value == None or value == "" or value == self.get(param) or (param == "destination" and value == "s"):
            return
        #print("\n\n>>> conversation, set: {}\n\n".format(self))
        self[param] = value
        self["event"][param] = value
        if param == "status":
            self["event"].push("CALL.UPDATE")

    def link_channel(self, participant, chan):
        #print(">>> link_channel participant: {}, chan: {}".format(participant, chan))
        if participant not in {"caller", "callee"}:
            return
        self[participant] = chan
        self.set_participant_info(participant,chan.get_info())
        # merge callee info looked up before callee channel has started
        if self.get("lookupInfo") != None and participant == "callee":
            self.set_participant_info("callee",self.get("lookupInfo"))
            self["lookupInfo"] = None

    def set_participant_info(self, participant, info):
        #print(">>> set_participant_info participant: {}, info: {}".format(participant,info))
        if participant not in {"caller", "callee"}:
            return
        if self.get(participant) == None:
            if participant == "callee":
                # store callee info, it will be processed upon callee channel start
                self["lookupInfo"] = info
        else:            
            for key,value in info.items():
                if value != "":
                    self[participant].set(key, value)
                    if self.get("event").get(participant) is None:
                        self["event"][participant] = {key:value}
                    else:
                        self["event"][participant][key] = value

    def set_flags(self, value):
        if self.get("flags") == None:
            self["flags"] = [value]
        else:
            self["flags"].append(value)
        if self.get("event").get("flags") == None:
            self["event"]["flags"] = [value]
        else:
            self["event"]["flags"].append(value)

    def set_app(self, app, destination):
        if not app in {"Dial", "Queue", "Page", "ConfBridge", "VoiceMail"}:
            return
        if self.get("app") == None:
            self["app"] = app
            self["event"]["app"] = app
            if self.get("destination") != destination:
                self.set("destination",destination)
                #print("\n\n>>> conversation: {}\n\n".format(self))
                self["event"].push("CALL.UPDATE")
        elif app == "VoiceMail":
            self["app"] = app
            self["event"]["app"] = app
            self.set_flags("voicemail")
            self.set("destination",destination)
        else:
            self["app"] = app
            self["event"]["app"] = app
            if self.get("destination") != destination:
                self.set("destination",destination)
                self.conv_split("dialplan")

    def conv_split(self, reason):
        self["status"] = "connecting"
        self["event"]["status"] = "connecting"
        self["splitReason"] = reason
        self["event"]["reason"] = reason
        #print("\n\n>>> conversation: {}\n\n".format(self))
        self["event"].push("CALL.SPLIT")

    def record_start(self):
        self["recording"] = "started"
        self["event"].push("CALL.RECORD_START")

    def record_stop(self, recording):
        self["recording"] = recording
        self["event"]["recording"] = recording
        self["event"].push("CALL.RECORD_STOP")
        if self.get("status") == "end":
            if self.get("ID") in ConversationsList.keys():
                del ConversationsList[self.get("ID")]

def add(conv):
    ConversationsList[conv["ID"]] = conv
    #print("\n\n>>> conversation: {}\n\n".format(conv))
    conv["event"].push("CALL.START")

def get(ID):
    return ConversationsList.get(ID)

def end(ID):
    conv = ConversationsList.get(ID)
    if conv == None:
        return
    #print("\n\n>>> conversation: {}\n\n".format(conv))
    conv["event"].push("CALL.END")
    conv["status"] = "end"
    if conv.get("recording") != "started" and ID in ConversationsList.keys():
        del ConversationsList[ID]

def print_currect_state():
    res = "ConversationsList:"
    for c in ConversationsList.keys():
        res += "\n\n{}".format(ConversationsList[c])
    print("===={}".format(res))