from datetime import datetime
import json
from wstat import Common, Aws, Channel


class Event(dict):
    def  __init__(self, ID):
        # print("====Init event " + str(ID))
        dict.__init__(self, pbx=Common.PBX_SERIAL)
        self["ID"] = ID
        self["name"] = None
        self["time"] = None
        self["status"] = None
        self["caller"] = None
        self["callee"] = None
        self["service"] = None
        self["app"] = None
        self["destination"] = None
        self["trunk"] = None
        self["queue"] = None
        self["tags"] = None
        self["flags"] = None
        self["season"] = None
        self["transferType"] = None
        self["recording"] = None
        self["mergeWith"] = None
        self["cause"] = None
        self["causeStr"] = None
        self["who"] = None

    # START/UPDATE
    def get_data(self):
        rv = {}
        for attr in {"status","caller","callee","service","destination","trunk","queue","tags","flags"}:
            value = self.get(attr)
            if value != None:
                rv[attr] = value
                # clean event data
                self[attr] = None
        return rv

    def get_data_split(self):
        rv = {}
        for attr in {"reason","service","transferType","mergeWith","status","caller","callee","destination","trunk","queue","tags","flags"}:
            value = self.get(attr)
            if value != None:
                rv[attr] = value
                self[attr] = None
        return rv

    def get_data_agent(self):
        rv = {}
        return rv

    def get_data_record_stop(self):
        rv = {}
        value = self.get("recording")
        if value != None:
            rv["fileName"] = value
            self["fileName"] = None
        return rv

    def get_data_end(self):
        rv = {}
        for attr in {"cause","causeStr","who"}:
            value = self.get(attr)
            if value != None:
                rv[attr] = value
                self[attr] = None
        return rv

    def push(self, name):
        #print("\n\n>>> Event.push, self: {}\n\n".format(self))
        self["time"]=int(datetime.timestamp(datetime.now())*1000)
        data = {}
        if name in {"CALL.START","CALL.UPDATE"}:
            data = self.get_data()
        elif name == "CALL.SPLIT":
            data = self.get_data_split()
        elif name == "CALL.RECORD_STOP":
            data = self.get_data_record_stop()
        elif name == "CALL.END":
            data = self.get_data_end()
        ev = {
            "id":self.get("ID"),
            "name":name,
            "time":self.get("time"),
            "pbx":self.get("pbx")
        }
        if len(data) > 0:
            ev["data"] = data
        payload = json.dumps(ev)
        print("\n\033[92m{},\033[0m".format(payload))
        payload_bytes = payload.encode('utf-8')
        put_response = Aws.KINESIS_CLIENT.put_record(
            StreamName=Aws.AWS_STREAM,
            Data=payload_bytes,
            PartitionKey=Common.PBX_SERIAL)
        #print("Response: " + str(put_response))